import os
from io import BytesIO

from functools import partial
from tempfile import TemporaryDirectory

import asyncio
import aiofiles

from shell_command_pool import ShellCommand, TaskPool, return_file_content
from shell_command_pool import run_sc

# comm = "./stiff -c stiff.conf orion_mosaic.fits -OUTFILE_NAME=test.tiff"


def hdu2s(hdu):
    o = BytesIO()
    hdu.writeto(o)
    r = o.getvalue()
    o.close()

    return r

STIFF_BIN_PATH = "stiff"
CONVERT_BIN_PATH = "convert"


async def hdu_byte_to_jpg(hdu_byte, bg, vmax, pool=None):

    # is there an async version of this?
    with TemporaryDirectory(suffix=None, prefix=None, dir=None) as tmpdirname:

        mosaic_name = os.path.join(tmpdirname, "mosaic.fits")
        tiff_name = os.path.join(tmpdirname, "mosaic.tiff")
        jpg_name = os.path.join(tmpdirname, "mosaic.jpg")

        async with aiofiles.open(mosaic_name,
                                 mode='bw') as f:
            await f.write(hdu_byte)

        sc1 = ShellCommand(STIFF_BIN_PATH,
                           mosaic_name,
                           "-SKY_TYPE", "MANUAL",
                           "-SKY_LEVEL", "{:f}".format(bg),
                           "-MAX_TYPE", "MANUAL",
                           "-MAX_LEVEL", "{:f}".format(vmax),
                           # "-MAX_TYPE=MANUAL",
                           # "-MAX_LEVEL=8000."
                           "-OUTFILE_NAME", tiff_name)

        sc2 = ShellCommand(CONVERT_BIN_PATH,
                           tiff_name, jpg_name)

        if pool is None:
            return await run_sc(sc1, sc2,
                                get_return=return_file_content(jpg_name),
                                cwd=tmpdirname)
        else:
            fut = await pool.push(sc1, sc2,
                                  get_return=return_file_content(jpg_name),
                                  cwd=tmpdirname)

            print("wait fot fut")
            return await fut


async def hdu_to_jpg(hdu, bg, vmax, pool=None):
        hdu_byte = hdu2s(hdu)
        return await hdu_byte_to_jpg(hdu_byte, bg, vmax, pool=pool)


async def main():

    import astropy.io.fits as pyfits
    hdu = pyfits.open("orion_mosaic.fits")[0]

    pool = TaskPool(run_sc, 4)
    pool.start_task_pool()

    # pool = None

    bg, vmax = 0, 5300
    r = await hdu_to_jpg(hdu, bg, vmax, pool=pool)

    if r is not None:
        open("test2.jpg", "wb").write(r)

    if pool is not None:
        print("wait fot join")
        await pool.join()
        await pool.gather()

if __name__ == '__main__':
    asyncio.run(main())
