import os
import sanic
from sanic import Sanic
from sanic.response import json
from asyncio import create_task
from sanic.log import logger
import sanic.response as response

import minio_cache_worker

from yaml import load

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

# hips
# import hips_survey_cached
# from pull_single_tile import HipsTileMetaFromURL, fetch_all_tiles_aiohttp

# from aio_run_stiff import hdu_byte_to_jpg
from pull_and_jpg import RenderedHipsTileFactory

MINIO_HOST_CONFIG_YAML = os.environ.get("MINIO_HOST_CONFIG_YAML",
                                        "minio_hosts.yaml")
MINIO_DEFAUL_HOST = os.environ.get("MINIO_DEFAUL_HOST",
                                   "kgmt")

# or "minio_hosts.yaml"
def load_minio_host_config(minio_host_config_yaml):
    _a = load(open(minio_host_config_yaml), Loader=Loader)

    content = _a

    clientDict = dict((a1["name"], dict(url=a1["url"],
                                        bucket_name=a1["bucket_name"]))
                           for a1 in _a["minio_hosts"])

    bucketDict = dict((a1["name"], a1["bucket_name"])
                           for a1 in _a["minio_hosts"])

    return dict(content=content,
                clientDict=clientDict,
                bucketDict=bucketDict)


minio_info = load_minio_host_config(MINIO_HOST_CONFIG_YAML)

factory = RenderedHipsTileFactory(minio_spec=minio_info["clientDict"][MINIO_DEFAUL_HOST],
                                  store_cache=True)

app = Sanic(name="hipsweb")

@app.listener('before_server_start')
async def start_app(*args, **kwargs):
    factory.start_pool()
    print("start app")
    # args[0].factory = RenderedHipsTileFactory()

@app.listener('before_server_stop')
async def stop_app(*args, **kwargs):
    await factory.destroy()
    print("finished")

@app.route('/ping')
async def ping(request):
    r = response.text(
        "Ping",
    )
    return r

@app.route('/ping_celery')
async def ping_celery(request):
    r = minio_cache_worker.ping.delay()
    try:
        body = r.get(timeout=1)
    except:
        body = None

    if body is not None:
        r = response.text(
            "ping celery succeeded",
            status=200
            )
    else:
        r = response.text(
            "ping celery failed",
            status=404,
        )

    return r

    # r = response.text(
    #     "Ping",
    # )
    # return r

@app.route('/custom/<survey_name>/<vmax:float>/<bg:float>/<norder>/<ndir>/<jpgname>')
async def get(request, survey_name, norder, ndir, jpgname, vmax, bg):
    # factory = app.factory
    fitsname = jpgname.replace(".jpg", ".fits")
    body = await factory.fetch_jpg(norder, ndir, fitsname,
                                   vmax=vmax, bg=bg)
    # t = create_task(factory.fetch_fits(norder, ndir, fitsname))
    # body = await t

    if body is not None:
        r = response.raw(
            body,
            status=200,
            headers=None,
            content_type='image/jpeg')
        return r
    else:
        r = response.text(
            "Error",
            status=404,
        )
        return r

if __name__ == "__main__":
    app.run(host="0.0.0.0", port=8000)

    # to test this, try something like below in your shell
    # wget localhost:8000/custom/S/1000/0/Norder8/Dir360000/Npix364882.jpg
