import time
import io

from celery import Celery

from minio import Minio
from minio.error import InvalidResponseError as ResponseError

# from minio.error import (InvalidResponseError as ResponseError,
#                          BucketAlreadyOwnedByYou,
#                          BucketAlreadyExists)

from yaml import load, dump

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper


s = """
minio_hosts:
  - name: local
    url: 'localhost:9000'
    access_key: 'hips-celery-worker'
    secret_key: 'hips-celery-worker-secret'
    secure: False
    bucket_name: 'hips-cache'
  - name: kgmt
    url: '210.110.233.67:9005'
    access_key: 'hips-celery-worker'
    secret_key: 'hips-celery-worker-secret'
    secure: False
    bucket_name: 'hips-cache'
"""

# data = load(stream, Loader=Loader)
# _a = load(s, Loader=Loader)
import os

## default config values
# CELERY_BROKER_URL='pyamqp://guest@rabbitmq//'
# CELERY_RESULT_BACKEND='rpc://'
# MINIO_HOST_CONFIG_YAML="minio_hosts.yaml"

# config from environments
CELERY_BROKER_URL = os.environ.get("CELERY_BROKER_URL",
                                   "pyamqp://guest@rabbitmq//")
CELERY_RESULT_BACKEND = os.environ.get("CELERY_RESULT_BACKEND",
                                       "rpc://")
MINIO_HOST_CONFIG_YAML = os.environ.get("MINIO_HOST_CONFIG_YAML",
                                        "minio_hosts.yaml")

# or "minio_hosts.yaml"
def load_minio_host_config(minio_host_config_yaml):
    _a = load(open(minio_host_config_yaml), Loader=Loader)

    content = _a

    clientDict = dict((a1["name"], Minio(a1["url"],
                                              access_key=a1["access_key"],
                                              secret_key=a1["secret_key"],
                                              secure=a1["secure"]))
                           for a1 in _a["minio_hosts"])

    bucketDict = dict((a1["name"], a1["bucket_name"])
                           for a1 in _a["minio_hosts"])

    return dict(content=content,
                clientDict=clientDict,
                bucketDict=bucketDict)


minio_info = load_minio_host_config(MINIO_HOST_CONFIG_YAML)

# minioClientDict = dict((local=Minio("localhost:9000",
#                                     access_key="hips-celery-worker",
#                                     secret_key="hips-celery-worker-secret"
#                                           access_key=a1["access_key"],
#                                           secret_key=a1["secret_key"],
#                                           secure=a1["secure"]))
#                        for a1 in _a["minio_hosts"])


# minioClientDict = {"local": Minio('localhost:9000',
#                                   access_key='hips-celery-worker',
#                                   secret_key='hips-celery-worker-secret',
#                                   secure=False),
#                    }

ErrorCode = {401: "Unregistered Minio Name"}

# bucket_name = "hips-cache"

# redis_url = 'redis://localhost:6379/0'
app = Celery('minio_cache_worker',
             backend=CELERY_RESULT_BACKEND,
             # backend='rpc://',
             # backend=redis_url,
             # task_serializer="msgpack",
             # broker=redis_url)
             # broker='pyamqp://guest@localhost//',
             broker=CELERY_BROKER_URL)
             # broker='pyamqp://guest@hipsweb-rmq//')

# app.conf['CELERY_TASK_SERIALIZER'] = 'msgpack'
# app.conf['CELERY_RESULT_SERIALIZER'] = 'pickle'
app.conf['CELERY_ACCEPT_CONTENT'] = ['json', 'msgpack']

@app.task(serializer="msgpack")
def ping():
    return True

@app.task(serializer="msgpack")
def get_config():
    r = dict(CELERY_BROKER_URL=CELERY_BROKER_URL,
             CELERY_RESULT_BACKEND=CELERY_RESULT_BACKEND,
             MINIO_HOST_CONFIG_YAML=MINIO_HOST_CONFIG_YAML,
             MINIO_HOST_CONFIG_YAML_CONTENT=minio_info["content"])

    return r


@app.task(serializer="msgpack")
def test_upload(minio_name):
    path = "test_123"
    raw_bite = u"642312"

    return push(minio_name, path, raw_bite, verify_upload=True)


@app.task(serializer="msgpack")
def push(minio_name, path, raw_bite, verify_upload=False):

    if minio_name not in minio_info["clientDict"]:
        return dict(status="error", error={"code": 401})

    minioClient = minio_info["clientDict"][minio_name]
    bucket_name = minio_info["bucketDict"][minio_name]

    data = io.BytesIO(raw_bite)
    try:
        minioClient.put_object(bucket_name, path,
                               data, len(raw_bite))

    except ResponseError as err:
        err_msg = "upload error"
        return {"return": "error", "msg": err_msg}

    if verify_upload:
        try:
            r = minioClient.get_object(bucket_name, path)
            assert r.read() == raw_bite
        except:
            err_msg = "upload error"
            return {"return": "error", "msg": err_msg}

    # time.sleep(1)
    print("pushed: {}".format(path))
    return 0


if __name__ == '__main__':
    push("kgmt", "test-file2.txt", b"123")
    # push("local", "test-file", b"123")
