import asyncio
import aiohttp

# import hips_survey_cached
from hips_cached.hips_cache_server import get_minio_cache_server
from hips_cached.hips_survey_cached import get_2MASS_K
from hips_cached.hips_tiles import HipsTileMetaFromURL
# from hips_cached.tile_meta import HipsTileMetaFromURL
from hips_cached.pull_single_tile import fetch_all_tiles_aiohttp

# from pull_single_tile import HipsTileMetaFromURL, fetch_all_tiles_aiohttp

from shell_command_pool import TaskPool, run_sc
from aio_run_stiff import hdu_byte_to_jpg

import minio_cache_worker

def push_to_minio_worker(fn, raw_data):
    minio_cache_worker.push.apply_async(("kgmt", fn, raw_data),
                                        ignore_result=True)


class RenderedHipsTileFactory():
    def __init__(self, minio_spec=None, store_cache=True):
        prefix = "2MASS_K/"

        if minio_spec is not None:
            minio_url = "http://" + minio_spec["url"]
            minio_bucket = minio_spec["bucket_name"]
            minio_url = "http://210.110.233.67:9005"
            minio_bucket = "hips-cache"

            local_cache_server = get_minio_cache_server(minio_url=minio_url,
                                                    minio_bucket=minio_bucket,
                                                    prefix=prefix)
            local_cache_server.set_push(push_to_minio_worker, prefix)
        else:
            local_cache_server = None

        self.hips_survey = get_2MASS_K(local_cache_server)

        # this is meant to an option to not store caches back to minio. Just
        # for testing purpose.
        self.store_cache = store_cache

    def start_pool(self):
        connector = aiohttp.TCPConnector()
        self.session = aiohttp.ClientSession(connector=connector)

        self.pool = TaskPool(run_sc, 4)
        self.pool.start_task_pool()

    async def fetch_fits(self, norder, dirname, fitsname):
        tile_url = "/".join([norder, dirname, fitsname])
        tile_meta = HipsTileMetaFromURL(tile_url)
        print(tile_meta.tile_default_url)

        tile_metas = [tile_meta]
        hips_survey = self.hips_survey
        session = self.session
        # pool = self.pool

        # note that n_parallel is not used if session is provided.
        tiles = await fetch_all_tiles_aiohttp(tile_metas, hips_survey,
                                              progress_bar=False,
                                              n_parallel=0,
                                              timeout=20, session=session)

        # bg, vmax = 0, 100
        if hasattr(tiles[0], "raw_data"):
            return tiles[0].raw_data

    async def fetch_jpg(self, norder, dirname, fitsname, vmax, bg):
        # if self.pool.tasks:
        #     pass
        # else:
        #     self.pool.start_task_pool()

        # tile_url = "Norder8/Dir360000/Npix364882.fits"
        try:
            # tile_url = "/".join([norder, dirname, fitsname])
            # tile_meta = HipsTileMetaFromURL(tile_url)
            # print(tile_meta.tile_default_url)

            # tile_metas = [tile_meta]
            # hips_survey = self.hips_survey
            # session = self.session
            # pool = self.pool

            # tiles = await fetch_all_tiles_aiohttp(tile_metas, hips_survey,
            #                                       progress_bar=True, n_parallel=1,
            #                                       timeout=20, session=session)

            # bg, vmax = 0, 100
            raw_data = await self.fetch_fits(norder, dirname, fitsname)
            # print("converting tiles", tiles)
            if raw_data is not None:
                # hasattr(tiles[0], "raw_data"):
                print("111")
                r = await hdu_byte_to_jpg(raw_data, bg, vmax,
                                          pool=self.pool)
            else:
                r = None
        except:
            import traceback
            traceback.print_exc()
            r = None

        return r

    async def destroy(self):
        print("wait fot join")
        await self.session.close()
        await self.pool.join()
        await self.pool.gather()

async def main():
    factory = RenderedHipsTileFactory()
    factory.start_pool()
    # tile_url = "Norder8/Dir360000/Npix364882.fits"

    r = await factory.fetch_jpg("Norder8", "Dir360000", "Npix364882.fits",
                                vmax=100, bg=0.)

    if r is not None:
        print("writing")
        open("test5.jpg", "wb").write(r)

    await factory.destroy()

if __name__ == '__main__':
    asyncio.run(main())
